/* ****************************************************************************
  This file is part of KBabel

  Copyright (C) 1999 by Matthias Kiefer
                            <matthias.kiefer@gmx.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************************** */

#include "catalogmanager.h"
#include "catalog.h"
#include "resources.h"
#include "kbabel.h"
#include "prefwidgets.h"
#include "msgfmt.h"

#include <kcmenumngr.h>
#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kapp.h>
#include <kaction.h>
#include <kwin.h>

#include <qfileinfo.h>
#include <qdir.h>
#include <qtimer.h>
#include <qbitmap.h>
#include <qwhatsthis.h>
#include <qmime.h>
#include <qheader.h>
#include <qdragobject.h>
#include <qmultilineedit.h>

#define COL_NAME 0
#define COL_MARKER 1
#define COL_FUZZY 2
#define COL_UNTRANS 3
#define COL_TOTAL 4
#define COL_REVISION 5
#define COL_TRANSLATOR 6


CatalogManagerView::CatalogManagerView(QWidget* parent,const char* name)
              : QListView(parent,name)
{
   _dirList.resize(200);
   _fileList.resize(500);
   _dirWatch=0;
   _logWindow=0;
   _logView=0;

   _pendingProcesses.setAutoDelete(true);

   _active=false;
   _stop=false;

   _updateTimer = new QTimer(this);
   connect(_updateTimer,SIGNAL(timeout()),this,SLOT(checkUpdate()));

   addColumn(i18n("Name"));
   addColumn(i18n("M"),25);
   setColumnAlignment(1,AlignCenter);
   addColumn(i18n("Fuzzy"));
   setColumnAlignment(1,AlignCenter);
   addColumn(i18n("Untranslated"));
   setColumnAlignment(2,AlignCenter);
   addColumn(i18n("Total"));
   setColumnAlignment(3,AlignCenter);
   addColumn(i18n("Last Revision"));
   addColumn(i18n("Last Translator"));


   header()->setMovingEnabled(false);
   setAllColumnsShowFocus(true);
   setSorting(0);

   if(KContextMenuManager::showOnButtonPress())
   {
      connect(this,SIGNAL(rightButtonPressed(QListViewItem*,const QPoint &, int))
           ,this, SLOT(showContentsMenu(QListViewItem*,const QPoint &, int)));
   }
   else
   {
      connect(this,SIGNAL(rightButtonClicked(QListViewItem*,const QPoint &, int))
           ,this, SLOT(showContentsMenu(QListViewItem*,const QPoint &, int)));
   }
   connect(this, SIGNAL(returnPressed(QListViewItem*))
           ,this, SLOT(activateItem(QListViewItem*)));
   connect(this, SIGNAL(doubleClicked(QListViewItem*))
           ,this, SLOT(activateItem(QListViewItem*)));
   connect(this,SIGNAL(selectionChanged(QListViewItem *)),this,SLOT(checkSelected(QListViewItem*)));

   _openAction = new KAction(i18n("&Open"),CTRL+Key_O,this,SLOT(slotOpenFile()),this);
   _openTemplateAction = new KAction(i18n("&Open Template"),Key_Space,this,SLOT(slotOpenTemplate()),this);
   _markAction = new KAction(i18n("&Toggle Marking"),CTRL+Key_M,this,SLOT(slotToggleMark()),this);
   _checkAction = new KAction(i18n("&Check Syntax"),CTRL+Key_C,this,SLOT(checkSyntax()),this);
   _deleteAction = new KAction(i18n("&Delete"),Key_Delete,this,SLOT(slotDeleteFile()),this);

   KAction* statisticAction = new KAction(i18n("&Statistics"),CTRL+Key_S
                                   ,this,SLOT(statistics()),this);
   KAction* toggleMarksAction = new KAction(i18n("Toggle Markings"),0
                                   ,this,SLOT(slotToggleMarksInDir()),this);
   KAction* clearMarksAction = new KAction(i18n("Remove Markings"),0
                                   ,this,SLOT(slotClearMarksInDir()),this);

   KAction* toggleAllMarksAction = new KAction(i18n("Toggle All Markings"),CTRL+Key_T
                                   ,this,SLOT(toggleAllMarks()),this);
   KAction* clearAllMarksAction = new KAction(i18n("Remove All Markings"),CTRL+Key_R
                                   ,this,SLOT(clearAllMarks()),this);


   _dirCommandsMenu = new QPopupMenu(this);
   connect(_dirCommandsMenu,SIGNAL(activated(int)),this,SLOT(slotDirCommand(int)));
   _fileCommandsMenu = new QPopupMenu(this);
   connect(_fileCommandsMenu,SIGNAL(activated(int)),this,SLOT(slotFileCommand(int)));

   _fileContentsMenu = new QPopupMenu(this);
   _openAction->plug(_fileContentsMenu);
   _openTemplateAction->plug(_fileContentsMenu);
   _fileContentsMenu->insertSeparator();
   _markAction->plug(_fileContentsMenu);
   toggleAllMarksAction->plug(_fileContentsMenu);
   clearAllMarksAction->plug(_fileContentsMenu);
   _fileContentsMenu->insertSeparator();
   _checkAction->plug(_fileContentsMenu);
   statisticAction->plug(_fileContentsMenu);
   _fileContentsMenu->insertItem(i18n("Commands"),_fileCommandsMenu);
   _fileContentsMenu->insertSeparator();
   _deleteAction->plug(_fileContentsMenu);


   _dirContentsMenu = new QPopupMenu(this);
   toggleMarksAction->plug(_dirContentsMenu);
   clearMarksAction->plug(_dirContentsMenu);
   toggleAllMarksAction->plug(_dirContentsMenu);
   clearAllMarksAction->plug(_dirContentsMenu);
   _dirContentsMenu->insertSeparator();
   _checkAction->plug(_dirContentsMenu);
   statisticAction->plug(_dirContentsMenu);
   _dirContentsMenu->insertItem(i18n("Commands"),_dirCommandsMenu);


   _logWindow = new KDialogBase(0,"log window",false,i18n("Log window")
                      ,KDialogBase::Close | KDialogBase::User1,KDialogBase::Close);
   _logWindow->setButtonText(KDialogBase::User1,i18n("C&lear"));
   _logWindow->setInitialSize(QSize(300,200));

   QWhatsThis::add(_logWindow,i18n("<qt><p><b>Log window</b></p>\n\
<p>In this window the output of the executed commands are shown.</p></qt>"));

   _logView = new QMultiLineEdit(_logWindow);
   _logView->setReadOnly(true);

   _logWindow->setMainWidget(_logView);

   connect(_logWindow,SIGNAL(user1Clicked()),_logView,SLOT(clear()));


   readMarker();

   QMimeSourceFactory::defaultFactory()->setPixmap("ok",BarIcon("ok"));
   QMimeSourceFactory::defaultFactory()->setPixmap("missing",BarIcon("missing"));
   QMimeSourceFactory::defaultFactory()->setPixmap("needwork",BarIcon("needwork"));
   QPixmap noPot(BarIcon("ok").size());
   noPot.fill();
   CatManListItem::paintExclamation(&noPot);
   QMimeSourceFactory::defaultFactory()->setPixmap("noPot",noPot);
   noPot=BarIcon("ok");
   CatManListItem::paintExclamation(&noPot);
   QMimeSourceFactory::defaultFactory()->setPixmap("noPot-ok",noPot);



   QWhatsThis::add(this,i18n("<qt><p><b>Catalog Manager</b></p>\n\
<p>The Catalog Manager merges two directories into one tree and displays all\n\
PO- and POT-files in these directories. This way you can easily see if a\n\
new template has been added or removed. Also some informations of the files\n\
are displayed in the Catalog Manager: total number, number of fuzzy and number of\n\
untranslated entries, the last revision and the last translator of the file.</p>\n\
<p>To make it easier for you to find files, that need work or that are missing,\n\
the status of the files is also displayed with an icon:\n\
<ul>\n\
<li><img source=\"ok\"/> The messages in this file are all translated.</li>\n\
<li><img source=\"needwork\"/> Some messages in this file are fuzzy or untranslated</li>\n\
<li><img source=\"missing\"/> This file does not exists in the directory of the PO-files.</li>\n\
</ul>\n\
If an icon is marked with this icon <img source=\"noPot\"/>, like\n\
<img source=\"noPot-ok\"/>, it indicates, that this file or directory does not exist in the\n\
directory of the POT-files.</p>\n\
<p>You can mark or unmark a file by selecting <b>Toggle Marking</b> in the context menu of a file.\n\
If you want to toggle or remove all markings in a directory, press the right mouse button\n\
over this directory and select <b>Toggle Markings</b> or <b>Remove Markings</b>.\n\
The markings are automatically saved when leaving KBabel.</p>\n\
<p>To open a file, double click on the file, or select it and press either return or Ctrl+O,\n\
or select <b>Open</b> from the context menu.</p>\n\
<p>You can configure the Catalog Manager either by clicking <b>Configure...</b> at the bottom\n\
or in section <b>Catalog Manager</b> of the preferences dialog.</p></qt>"));

   restoreView();

   setAcceptDrops(true); // just to get the drag displayed immediately
}

CatalogManagerView::~CatalogManagerView()
{
  if(_dirWatch)
      delete _dirWatch;

  saveMarker();
  saveView();

  if(_active)
     stop();

  if(_settings.killCmdOnExit)
  {
     KProcess* proc;
     for ( proc=_pendingProcesses.first(); proc != 0; proc=_pendingProcesses.next() )
     {
        proc->kill(SIGKILL);
     }
  }

  delete _logWindow;
}


void CatalogManagerView::saveView() const
{     /*
    KConfig* config=KGlobal::config();
    KConfigGroupSaver cs(config,"CatalogManager");
    */
}


void CatalogManagerView::restoreView()
{     /*
    KConfig* config=KGlobal::config();
    KConfigGroupSaver cs(config,"CatalogManager");
    */
}


void CatalogManagerView::checkUpdate()
{
   pause(true);

   CatManListItem* item = _dirList["/"];

   if(item)
   {
      item=(CatManListItem*)item->itemBelow();
   }
   while(item)
   {
      item->checkUpdate();
      item=(CatManListItem*)item->itemBelow();
   }

   pause(false);
}



void CatalogManagerView::pause(bool flag)
{
   if(flag)
   {
      _updateTimer->stop();
   }
   else
   {
      _updateTimer->start(60000);
   }
}


void CatalogManagerView::stop()
{
   _stop=true;
}

void CatalogManagerView::clear()
{
   pause(true);

   // first clear up
   if(_dirWatch)
      delete _dirWatch;

   _dirWatch= new KDirWatch(1000);
   connect(_dirWatch,SIGNAL(deleted(const QString&)),this,SLOT(directoryDeleted(const QString&)));
   connect(_dirWatch,SIGNAL(dirty(const QString&)),this,SLOT(directoryChanged(const QString&)));

   _dirList.clear();
   _fileList.clear();

   QListView::clear();
}

void CatalogManagerView::toggleAllMarks()
{
   _markerList.clear();

   QListViewItemIterator it( this );
   CatManListItem* item;

   for ( ; it.current(); ++it )
   {
      item = (CatManListItem*) it.current();
      if(item->isFile())
      {
         bool wasMarked=item->marked();
         item->setMarked(!wasMarked);
         if(!wasMarked)
         {
            _markerList.append(item->package());
         }
         else
         {
            _markerList.remove(item->package());
         }
      }
   }
}

void CatalogManagerView::clearAllMarks()
{
   _markerList.clear();
   QDictIterator<CatManListItem> it( _fileList ); // iterator for dict

   while ( it.current() )
   {
      CatManListItem* item=it.current();

      if(item->marked())
         _markerList.remove(item->package());

      item->setMarked(false);
      ++it;
   }

}

void CatalogManagerView::statistics()
{
   CatManListItem* i=(CatManListItem*) currentItem();

   if(!i)
     i=(CatManListItem*)_dirList["/"];

   if(isActive() && i->isDir())
   {
      if(KMessageBox::warningContinueCancel(this
         ,i18n("The Catalog Manager is still updating information about the files!\n\
If you continue, it tries to update all necessary files, but this can take a time and\n\
it may lead to wrong results. Please wait until all files are updated."),i18n("Warning")
         ,i18n("Continue")) == KMessageBox::Cancel)
      {
        return;
      }
   }

   QString msg;

   if(i->isFile())
   {
      i->checkUpdate();

      int totalMsgid = i->total();
      int fuzzy      = i->fuzzy();
      int untrans    = i->untranslated();

      if(totalMsgid==0)
         msg=i18n("Sorry, was not able to read information about the file's contents.");
      else
      {
         msg=i18n("Statistics for %1:\n\n").arg( i->package(false) + ".po");
         msg+=i18n("Number of messages: %1\n").arg( totalMsgid );

         double percent=((double)fuzzy*100)/totalMsgid;
         msg+=i18n("Fuzzy: %1% (%2)\n").arg(percent,0,'f',2).arg( fuzzy );

         percent=( (double)untrans*100 )/totalMsgid;
         msg+=i18n("Untranslated: %1% (%2)\n").arg(percent,0,'f',2).arg( untrans );
      }
   }
   else
   {
      int totalPackages=0;
      int totalPo=0;
      int totalNoPot=0;
      int needworkPo=0;
      int totalMsgid=0;
      int totalFuzzy=0;
      int totalUntranslated=0;

      QStringList childrenList=i->allChildrenList(true);

      QStringList::Iterator it;
      for( it = childrenList.begin(); it != childrenList.end(); ++it )
      {
         CatManListItem* item = _fileList[(*it)];

         /*
         KASSERT1(item,KDEBUG_FATAL,KBABEL_CATMAN,"CatalogManagerView::statistics: item not in list %s"
                  ,(*it).ascii());
           */
         // be sure, that the information is updated
         item->checkUpdate();

         totalPackages++;

         int fuzzy=item->fuzzy();
         int total=item->total();
         int untrans=item->untranslated();

         if(item->hasPo())
            totalPo++;

         if(!item->hasPot())
            totalNoPot++;


         if(fuzzy || untrans)
            needworkPo++;

         totalMsgid+=total;
         totalFuzzy+=fuzzy;
         totalUntranslated+=untrans;
      }

      double percent;

      QString name=i->package(false);
      if(name.isEmpty())
         name=i18n("all");

      msg=i18n("Statistics for %1:\n\n").arg(name);

      msg+=i18n("Number of packages: %1\n").arg(totalPackages);

      percent=100-((double)needworkPo*100)/totalPackages;
      msg+=i18n("Complete translated: %1% (%2)\n").arg(percent,0,'f',2).arg(totalPackages-needworkPo);

      percent=100-((double)totalPo*100)/totalPackages;
      msg+=i18n("Only template available: %1% (%2)\n").arg(percent,0,'f',2).arg(totalPackages-totalPo);
      percent=((double)totalNoPot*100)/totalPackages;
      msg+=i18n("Only PO-file available: %1% (%2)\n").arg(percent,0,'f',2).arg(totalNoPot);

      msg+=i18n("\nNumber of messages: %1\n").arg(totalMsgid);

      percent=((double)totalFuzzy*100)/totalMsgid;
      msg+=i18n("Fuzzy: %1% (%2)\n").arg(percent,0,'f',2).arg(totalFuzzy);

      percent=((double)totalUntranslated*100)/totalMsgid;
      msg+=i18n("Untranslated: %1% (%2)\n").arg(percent,0,'f',2).arg(totalUntranslated);

   }

   KMessageBox::information(this,msg,i18n("Statistics"));
}

void CatalogManagerView::checkSyntax()
{
   CatManListItem* item=(CatManListItem*) currentItem();

   if(!item)
      item=(CatManListItem*) _dirList["/"];

   if(item->isFile())
   {
      if(!item->hasPo())
         return;

      Msgfmt::Status status;
      QString output;
      Msgfmt msgfmt;

      status=msgfmt.checkSyntax(item->poFile(),output);

      switch(status)
      {
         case Msgfmt::Ok:
         {
            KMessageBox::information(this,i18n("The file is syntactical correct!\n\nOutput of \"msgfmt --statistics\":")+"\n"+output);
            break;
         }
         case Msgfmt::SyntaxError:
         {
            KMessageBox::information(this,i18n("The file has syntax errors!\n\nOutput of \"msgfmt --statistics\":")+"\n"+output);
            break;
         }
         case Msgfmt::Error:
         {
            KMessageBox::error(this,i18n("An error occured while processing \"msgfmt --statistics\""));
            break;
         }
         case Msgfmt::NoExecutable:
         {
            KMessageBox::sorry(this,i18n("Can't execute msgfmt.\nPlease make sure you have msgfmt in your PATH!"));
            break;
         }
      }

   }
   else
   {
      Msgfmt::Status status;
      QString output;
      Msgfmt msgfmt;

      status=msgfmt.checkSyntaxInDir(item->poFile(), "*.po", output);

      QString name=item->package(false);

      switch(status)
      {
         case Msgfmt::Ok:
         {
            QString msg;
            if(!name.isEmpty())
            {
               msg=i18n("All files in directory %1 are syntactical correct!\n\n\
Output of \"msgfmt --statistics\":\n").arg(name)+output;
            }
            else
            {
               msg=i18n("All files in the base directory are syntactical correct!\n\n\
Output of \"msgfmt --statistics\":\n")+output;
            }
            KMessageBox::information(this,msg);
            break;
         }
         case Msgfmt::SyntaxError:
         {
            QString msg;
            if(!name.isEmpty())
            {
               msg=i18n("At least one file in directory %1 has syntax errors!\n\n\
Output of \"msgfmt --statistics\":\n").arg(name)+output;
            }
            else
            {
               msg=i18n("At least one file in the base directory has syntax errors!\n\n\
Output of \"msgfmt --statistics\":\n")+output;
            }
            KMessageBox::information(this,msg);
            break;
         }
         case Msgfmt::Error:
         {
            QString msg;
            if(!name.isEmpty())
            {
              msg=i18n("An error occured while processing \"msgfmt --statistics *.po\"\n\
in directory %1").arg(name);
            }
            else
            {
              msg=i18n("An error occured while processing \"msgfmt --statistics *.po\"\n\
in the base directory");
            }
            KMessageBox::error(this,msg);
            break;
         }
         case Msgfmt::NoExecutable:
         {
            KMessageBox::sorry(this,i18n("Can't execute msgfmt.\nPlease make sure you have msgfmt in your PATH!"));
            break;
         }
      }
   }
}


void CatalogManagerView::showLog()
{
   _logWindow->show();
}


void CatalogManagerView::showContentsMenu(QListViewItem *i, const QPoint &point, int)
{
   CatManListItem* item = (CatManListItem*) i;

   if(!item)
      return;

   if(item->isDir())
   {
      item->setOpen(true);
      _dirContentsMenu->exec(point);
   }
   else
   {
      _checkAction->setEnabled(item->hasPo());

      _fileContentsMenu->exec(point);

      _checkAction->setEnabled(true);
   }
}

void CatalogManagerView::checkSelected(QListViewItem *i)
{
   CatManListItem* item=(CatManListItem*)i;

   if(!item)
      return;

   _openTemplateAction->setEnabled(item->hasPot());
   _deleteAction->setEnabled((!item->hasPot() && item->hasPo()));
   _markAction->setEnabled((item->isFile()));
}

void CatalogManagerView::activateItem(QListViewItem *)
{
   CatManListItem* item=(CatManListItem*) selectedItem();

   if(!item)
      return;

   QString filename;
   if(item->hasPo())
   {
      emit openFile(item->poFile());
   }
   else if(item->hasPot())
   {
      emit openTemplate(item->potFile(),item->poFile());
   }
   else
   {
      kDebugError(KBABEL_CATMAN,"CatalogManagerView::activateItem: item has no file?");
   }

}

void CatalogManagerView::slotOpenFile()
{
   CatManListItem* item=(CatManListItem*) selectedItem();

   if(item)
   {
      activateItem(item);
   }
}

void CatalogManagerView::slotOpenTemplate()
{
   CatManListItem* item=(CatManListItem*) selectedItem();

   if(item)
   {
      emit openFile(item->potFile());
   }
}

void CatalogManagerView::slotDeleteFile()
{

   CatManListItem* item=(CatManListItem*) selectedItem();

   if(item && item->hasPo() && !item->hasPot())
   {
      QString msg=i18n("Do you really want to delete the file\n%1 ?").arg(item->poFile());
      if(KMessageBox::warningContinueCancel(this,msg,i18n("Warning"),i18n("Delete"))== KMessageBox::Continue)
      {
         if(!QFile::remove(item->poFile()))
         {
             KMessageBox::sorry(this,i18n("Was not able to delete the file %1 !").arg(item->poFile()));
         }
      }
   }
}

void CatalogManagerView::slotToggleMark()
{
   CatManListItem* item=(CatManListItem*) selectedItem();

   if(item && !item->isDir())
   {
      bool wasMarked=item->marked();
      item->setMarked(!wasMarked);

      if(wasMarked)
      {
         _markerList.remove(item->package());
      }
      else
      {
         _markerList.append(item->package());
      }
   }
}

void CatalogManagerView::slotToggleMarksInDir()
{
   CatManListItem* i=(CatManListItem*) currentItem();

   if(i && i->isDir())
   {
      QStringList contentList = i->contentsList(true);

      QStringList::Iterator it;
      for( it = contentList.begin(); it != contentList.end(); ++it )
      {
         CatManListItem* item = _fileList[(*it)];

         KASSERT(item,KDEBUG_FATAL,KBABEL_CATMAN,"CatalogManagerView::slotToggleMarkInDir: item not in list");


         bool wasMarked=item->marked();
         item->setMarked(!wasMarked);

         if(wasMarked)
         {
            _markerList.remove(item->package());
         }
         else
         {
            _markerList.append(item->package());
         }
      }
   }
}


void CatalogManagerView::slotClearMarksInDir()
{
   CatManListItem* i=(CatManListItem*) currentItem();

   if(i && i->isDir())
   {
      QStringList contentList=i->contentsList(true);

      QStringList::Iterator it;
      for( it = contentList.begin(); it != contentList.end(); ++it )
      {
         CatManListItem* item = _fileList[(*it)];

         KASSERT(item,KDEBUG_FATAL,KBABEL_CATMAN,"CatalogManagerView::slotClearMarkInDir: item not in list");


         if(item->marked())
         {
            _markerList.remove(item->package());
         }
         item->setMarked(false);
      }
   }
}


void CatalogManagerView::slotDirCommand(int index)
{
   CatManListItem* item=(CatManListItem*) currentItem();


   if(index>=0 && item && item->isDir())
   {
      QString cmd=_settings.dirCommands[(uint)index];
      cmd.replace(QRegExp("@PACKAGE@"),item->name());
      cmd.replace(QRegExp("@PODIR@"),item->poFile());
      cmd.replace(QRegExp("@POTDIR@"),item->potFile());

      qDebug(cmd);

      KProcess* proc = new KShellProcess();
      _pendingProcesses.append(proc);

      connect( proc,SIGNAL( processExited(KProcess *) ), this
               ,SLOT( processEnded(KProcess*) ) );
      connect( proc,SIGNAL( receivedStdout(KProcess*,char*,int) ), this
               ,SLOT( showOutput(KProcess*,char*,int) ) );
      connect( proc,SIGNAL( receivedStderr(KProcess*,char*,int) ), this
               ,SLOT( showOutput(KProcess*,char*,int) ) );

      *proc << "cd" << item->poFile() << ";" << cmd;
      proc->start(KProcess::NotifyOnExit,KProcess::AllOutput);
   }
}

void CatalogManagerView::slotFileCommand(int index)
{
   CatManListItem* item=(CatManListItem*) currentItem();


   if(index>=0 && item && item->isFile())
   {
      CatManListItem* parent = (CatManListItem*)item->parent();

      QString cmd=_settings.fileCommands[(uint)index];
      cmd.replace(QRegExp("@PACKAGE@"),item->name());
      cmd.replace(QRegExp("@POFILE@"),item->poFile());
      cmd.replace(QRegExp("@POTFILE@"),item->potFile());
      cmd.replace(QRegExp("@PODIR@"),parent->poFile());
      cmd.replace(QRegExp("@POTDIR@"),parent->potFile());

      qDebug(cmd);

      KProcess* proc = new KShellProcess();
      _pendingProcesses.append(proc);

      connect( proc,SIGNAL( processExited(KProcess *) ), this
               ,SLOT( processEnded(KProcess*) ) );
      connect( proc,SIGNAL( receivedStdout(KProcess*,char*,int) ), this
               ,SLOT( showOutput(KProcess*,char*,int) ) );
      connect( proc,SIGNAL( receivedStderr(KProcess*,char*,int) ), this
               ,SLOT( showOutput(KProcess*,char*,int) ) );

      *proc << "cd" << parent->poFile() << ";" << cmd;
      proc->start(KProcess::NotifyOnExit,KProcess::AllOutput);
   }

}


void CatalogManagerView::updateFile(QString fileWithPath)
{
   QString relFile;
   if(fileWithPath.contains(QRegExp("^"+_settings.poBaseDir)))
   {
      relFile=fileWithPath.right(fileWithPath.length()-_settings.poBaseDir.length());
   }
   else if(fileWithPath.contains(QRegExp("^"+_settings.potBaseDir)))
   {
      relFile=fileWithPath.right(fileWithPath.length()-_settings.potBaseDir.length());
   }
   else
   {
      return;
   }

   if(relFile.right(4)==".pot")
   {
      relFile.truncate(relFile.length()-4);
   }
   else if(relFile.right(3)==".po")
   {
      relFile.truncate(relFile.length()-3);
   }

   CatManListItem* item=_fileList[relFile];

   if(item)
   {
      item->checkUpdate();
   }

}

void CatalogManagerView::buildTree()
{
   clear();

   if(isActive())
      return;

   _active=true;
   _stop=false;


   CatManListItem* root = new CatManListItem(this,_settings.poBaseDir,_settings.potBaseDir);
   _dirList.insert("/",root);
   root->setSelectable(false);

   bool haveTemplateDir=true;  // flag, if the potBaseDir is valid

   QFileInfo fileInfo(_settings.poBaseDir);
   if(!fileInfo.isDir())
   {
      KMessageBox::error(this,i18n("You don't have specified a valid directory\n\
for the base directory of the PO-files:\n%1\n\n\
Please check your settings in the preferences dialog!").arg(_settings.poBaseDir));

      _active=false;
      return;
   }

   fileInfo.setFile(_settings.potBaseDir);
   if(!fileInfo.isDir())
   {
      haveTemplateDir=false;
      KMessageBox::error(this,i18n("You don't have specified a valid directory\n\
for the base directory of the PO-templates:\n%1\n\n\
Please check your settings in the preferences dialog!").arg(_settings.potBaseDir));
   }




   //"/" is the root item
   buildDir("/",true);   // build dir without updating the items...
   _dirWatch->addDir(_settings.poBaseDir);
   _dirWatch->addDir(_settings.potBaseDir);


   root->setOpen(true);

   QDictIterator<CatManListItem> it( _fileList ); // iterator for dict

   while ( it.current() && !_stop)
   {
      it.current()->checkUpdate();

      kapp->processEvents();

      ++it;
   }

   _dirWatch->startScan();
   pause(false);

   _active=false;
}

bool CatalogManagerView::buildDir(QString relDir,bool fast)
{
   bool potHasFiles=buildDir(_settings.potBaseDir,relDir,".pot",fast);
   bool poHasFiles=buildDir(_settings.poBaseDir,relDir,".po",fast);

   return (poHasFiles | potHasFiles);
}


bool CatalogManagerView::buildDir(const QString& baseDir,const QString& relDir
                                                , const QString extension , bool fast)
{
      bool havePoFiles=false;

      CatManListItem* thisItem=_dirList[relDir];
      if(!thisItem)
      {
         kDebugFatal(KBABEL_CATMAN,"null pointer to this item");
         return false;
      }

      QString poBaseDir=_settings.poBaseDir;
      QString potBaseDir=_settings.potBaseDir;

      // traverse directory in poBaseDir
      QDir dir(baseDir+relDir);
      QStringList entryList=dir.entryList("*"+extension,QDir::Files,QDir::Name);

      QStringList::Iterator it;

      for (  it = entryList.begin(); it != entryList.end(); ++it )
      {
          havePoFiles=true;

          QString file=relDir+(*it);
          file.replace(QRegExp(extension+"$"),"");
          CatManListItem* item = _fileList[file];
          if(!item)
          {
             item = new CatManListItem(thisItem,poBaseDir+file+".po",potBaseDir+file+".pot",file);
             _fileList.insert(file,item);

             if(_markerList.contains(file))
             {
                item->setMarked(true);
             }

             if(!fast)
             {
                item->checkUpdate();
             }
          }
      }

      entryList=dir.entryList(QDir::Dirs,QDir::Name);

      for (  it = entryList.begin(); it != entryList.end(); ++it )
      {
          kapp->processEvents();

          if((*it)=="." || (*it)=="..")
          {
             continue;
          }

          QString subDir=relDir+(*it)+"/";
          if(!_dirWatch->contains(baseDir+subDir))
          {
              _dirWatch->addDir(baseDir+subDir);
          }

          bool otherHasFiles=true;

          CatManListItem* item = _dirList[subDir];
          if(!item)
          {
             item = new CatManListItem(thisItem,poBaseDir+subDir,potBaseDir+subDir,subDir);
             _dirList.insert(subDir,item);

             otherHasFiles=false;
          }

          // recursive call
          if(!buildDir(baseDir,subDir,extension,fast) && !otherHasFiles)
          {
             kDebugInfo(KBABEL_CATMAN,"skipping %s",subDir.ascii());
             _dirList.remove(subDir);
             delete item;
          }
          else
             havePoFiles=true;

      } // end looking up directories in po base dir

      return havePoFiles;
}


void CatalogManagerView::updateDir(QString relDir)
{
      bool havePoFiles=false;

      CatManListItem* thisItem=_dirList[relDir];
      if(!thisItem)
      {
         kDebugFatal(KBABEL_CATMAN,"null pointer to this item");
         return;
      }

      QStringList contentList = thisItem->contentsList(true);

      QString poBaseDir=_settings.poBaseDir;
      QString potBaseDir=_settings.potBaseDir;

      // first lookup template directory
      QDir dir(potBaseDir+relDir);
      QStringList entryList=dir.entryList("*.pot",QDir::Files,QDir::Name);

      QStringList::Iterator it;

      for (  it = entryList.begin(); it != entryList.end(); ++it )
      {
          havePoFiles=true;

          QString file=relDir+(*it);
          file.replace(QRegExp(".pot$"),"");
          CatManListItem* item = _fileList[file];
          if(!item)
          {
             item = new CatManListItem(thisItem,poBaseDir+file+".po",potBaseDir+file+".pot",file);
             _fileList.insert(file,item);

             if(_markerList.contains(file))
             {
                item->setMarked(true);
             }

             item->checkUpdate();
          }

          contentList.remove(file);
      }

      entryList=dir.entryList(QDir::Dirs,QDir::Name);

      for (  it = entryList.begin(); it != entryList.end(); ++it )
      {
          kapp->processEvents();

          if((*it)=="." || (*it)=="..")
          {
             continue;
          }

          bool newDirAdded=false;

          QString subDir=relDir+(*it)+"/";
          if(!_dirWatch->contains(potBaseDir+subDir))
          {
              _dirWatch->addDir(potBaseDir+subDir);

              newDirAdded=true;
          }

          CatManListItem* item = _dirList[subDir];
          if(!item && newDirAdded)
          {
             item = new CatManListItem(thisItem,poBaseDir+subDir,potBaseDir+subDir,subDir);
             _dirList.insert(subDir,item);

              if(!buildDir(subDir,false))
              {
                   kDebugInfo(KBABEL_CATMAN,"skipping %s",subDir.ascii());
                   _dirList.remove(subDir);
                   delete item;
                   item=0;
              }
          }

          // if directory was already here, but no item -> directory contains no files
          if(item && !newDirAdded)
          {
             havePoFiles=true;
          }

      } // end looking up directories in template dir

      // now traverse directory in poBaseDir
      dir.setPath(poBaseDir+relDir);
      entryList=dir.entryList("*.po",QDir::Files,QDir::Name);

      for (  it = entryList.begin(); it != entryList.end(); ++it )
      {
          havePoFiles=true;

          QString file=relDir+(*it);
          file.replace(QRegExp(".po$"),"");
          CatManListItem* item = _fileList[file];
          if(!item)
          {
             item = new CatManListItem(thisItem,poBaseDir+file+".po",potBaseDir+file+".pot",file);
             _fileList.insert(file,item);

             if(_markerList.contains(file))
             {
                item->setMarked(true);
             }

             item->checkUpdate();
          }

          contentList.remove(file);
      }

      entryList=dir.entryList(QDir::Dirs,QDir::Name);

      for (  it = entryList.begin(); it != entryList.end(); ++it )
      {
          kapp->processEvents();

          if((*it)=="." || (*it)=="..")
          {
             continue;
          }

          bool newDirAdded=false;

          QString subDir=relDir+(*it)+"/";
          if(!_dirWatch->contains(poBaseDir+subDir))
          {
               _dirWatch->addDir(poBaseDir+subDir);
               newDirAdded=true;
          }

          CatManListItem* item = _dirList[subDir];

          bool templateHasFiles=(bool)item;

          if(!item && newDirAdded)
          {
             item = new CatManListItem(thisItem,poBaseDir+subDir,potBaseDir+subDir,subDir);
             _dirList.insert(subDir,item);

              if(!buildDir(subDir,false) && !templateHasFiles)
              {
                   kDebugInfo(KBABEL_CATMAN,"skipping %s",subDir.ascii());
                   _dirList.remove(subDir);
                   delete item;
              }
          }

          // if directory was already here, but no item -> directory contains no files
          if(item && !newDirAdded)
          {
             havePoFiles=true;
          }


      } // end looking up directories in po base dir


      // check, if something in the directory has been deleted
      // but only if we traversed also the template directory
      if(contentList.count()>0)
      {
          QStringList::Iterator it;
          for( it = contentList.begin(); it != contentList.end(); ++it )
          {
             QFileInfo po(poBaseDir+(*it));
             QFileInfo pot(potBaseDir+(*it));

             if(!po.exists() && !pot.exists())
             {
                CatManListItem* item = _fileList[(*it)];
                if(item)
                {
                   if(item->marked())
                      _markerList.remove(item->package());

                   _fileList.remove((*it));
                   delete item;
                }
             }
          }
      }

      if(!havePoFiles)
      {
         _dirList.remove(relDir);
         delete thisItem;

          // if this directory has to be removed, check, if the parent directory has
          // to be removed too
          int index=relDir.findRev("/",relDir.length()-2);
          if(index<0)
          {
            relDir="/";
          }
          relDir=relDir.left(index+1);
          updateDir(relDir);
      }
}

void CatalogManagerView::directoryChanged(const QString& dir)
{
   QString relDir;
   if(dir.contains(QRegExp("^"+_settings.poBaseDir)))
   {
      relDir=dir.right(dir.length()-_settings.poBaseDir.length());
   }
   else if(dir.contains(QRegExp("^"+_settings.potBaseDir)))
   {
      relDir=dir.right(dir.length()-_settings.potBaseDir.length());
   }

   if(relDir.right(1)!="/")
   {
      relDir+="/";
   }

   kDebugInfo(KBABEL_CATMAN,"directory changed: %s",relDir.ascii());

   QFileInfo fileInfo(_settings.potBaseDir);

   CatManListItem* thisItem=_dirList[relDir];
   if(!thisItem)
   {
      // if this item is not in the list search for next existing parent item
      QString prevRelDir;
      do
      {
          prevRelDir=relDir;
          int index=relDir.findRev("/",relDir.length()-2);
          if(index<0)
          {
            relDir="/";
          }
          relDir=relDir.left(index+1);

          thisItem=_dirList[relDir];
      }
      while(relDir!="/" && !thisItem);

      if(!thisItem)
      {
         kDebugFatal(KBABEL_CATMAN,"null pointer to this item: %s",relDir.ascii());
         return;
      }
      else
      {
          // if a parent item dir is found, creat the needed item in this dir
          // and build the tree from this item on
          qDebug("building dir: "+prevRelDir);
          CatManListItem* item = new CatManListItem(thisItem,_settings.poBaseDir+prevRelDir
                     ,_settings.potBaseDir+prevRelDir,prevRelDir);
          _dirList.insert(prevRelDir,item);


          if(!buildDir(prevRelDir,false))
          {
             _dirList.remove(prevRelDir);
             delete item;
          }
      }
   }
   else
   {
      updateDir(relDir);
   }
}


void CatalogManagerView::directoryDeleted(const QString& dir)
{
   QString relDir;
   if(dir.contains(QRegExp("^"+_settings.poBaseDir)))
   {
      relDir=dir.right(dir.length()-_settings.poBaseDir.length());
   }
   else if(dir.contains(QRegExp("^"+_settings.potBaseDir)))
   {
      relDir=dir.right(dir.length()-_settings.potBaseDir.length());
   }

   if(relDir.right(1)!="/")
   {
      relDir+="/";
   }

   kDebugInfo(KBABEL_CATMAN,"directory deleted: %s",relDir.ascii());

   CatManListItem* thisItem=_dirList[relDir];
   if(thisItem)
   {
      QStringList childList = thisItem->allChildrenList();

      CatManListItem* item;
      QStringList::Iterator it;
      for( it = childList.begin();it !=  childList.end(); ++it )
      {
         item=_fileList[(*it)];
         if(item)
         {
            _fileList.remove((*it));
            delete item;
         }
         else
         {
            item=_dirList[(*it)];
            if(item)
            {
               _dirList.remove((*it));
               delete item;
            }
            else
            {
               kDebugFatal(KBABEL_CATMAN,"directoryDeleted: don't have item %s",(*it).ascii());
            }
         }

      }

      _dirList.remove(relDir);
      delete thisItem;

   }
}



void CatalogManagerView::setSettings(CatManSettings newSettings)
{
   CatManSettings oldSettings=_settings;
   _settings=newSettings;

   if(_settings.poBaseDir.right(1)=="/")
      _settings.poBaseDir.truncate(_settings.poBaseDir.length()-1);
   if(_settings.potBaseDir.right(1)=="/")
      _settings.potBaseDir.truncate(_settings.potBaseDir.length()-1);

   _dirCommandsMenu->clear();
   int counter=0;
   for ( QStringList::Iterator it = _settings.dirCommandNames.begin()
                        ; it != _settings.dirCommandNames.end(); ++it )
   {
      _dirCommandsMenu->insertItem((*it),counter);
      counter++;
   }
   _dirCommandsMenu->insertSeparator();
   _dirCommandsMenu->insertItem(i18n("Log Window"),this,SLOT(showLog()));

   _fileCommandsMenu->clear();
   counter=0;
   for ( QStringList::Iterator it = _settings.fileCommandNames.begin()
                        ; it != _settings.fileCommandNames.end(); ++it )
   {
      _fileCommandsMenu->insertItem((*it),counter);
      counter++;
   }
   _fileCommandsMenu->insertSeparator();
   _fileCommandsMenu->insertItem(i18n("Log Window"),this,SLOT(showLog()));


   bool pathChanged=false;
   if(oldSettings.poBaseDir!=_settings.poBaseDir)
   {
      pathChanged=true;
   }
   else if(oldSettings.potBaseDir!=_settings.potBaseDir)
   {
      pathChanged=true;
   }
   if(pathChanged)
   {
      QTimer::singleShot(10,this,SLOT(buildTree()));
   }

   emit settingsChanged(_settings);
}


CatManSettings CatalogManagerView::settings() const
{
   return _settings;
}


void CatalogManagerView::hideEvent(QHideEvent*)
{
   pause(true);

   if(_dirWatch)
      _dirWatch->stopScan();
}

void CatalogManagerView::showEvent(QShowEvent*)
{
   QTimer::singleShot(1,this,SLOT(checkUpdate()));

   pause(false);

   if(_dirWatch)
      _dirWatch->startScan(true);
}

void CatalogManagerView::contentsMousePressEvent(QMouseEvent* event)
{
   if(event->button() == LeftButton)
      _pressPos=event->pos();

   QListView::contentsMousePressEvent( event );
}

void CatalogManagerView::contentsMouseMoveEvent(QMouseEvent* event)
{
   if(event->state() & LeftButton)
   {
      int delay = KGlobal::dndEventDelay();
      if(QABS( event->pos().x() - _pressPos.x() ) >= delay ||
         QABS( event->pos().y() - _pressPos.y() ) >= delay)
      {
         CatManListItem* item = (CatManListItem*)itemAt(contentsToViewport(_pressPos));
         if(item && item->isFile())
         {
            // always add the po-file and if existing the pot-file to the drag and
            // let the user decide what to do, when dropping into kbabel
            QStrList uri;
            uri.append(QUriDrag::localFileToUri(item->poFile()));
            if(item->hasPot())
               uri.append(QUriDrag::localFileToUri(item->potFile()));

            QUriDrag* drag = new QUriDrag(uri,this);
            QPixmap icon=KGlobal::iconLoader()->loadIcon("mimetypes/txt");
            drag->setPixmap(icon,QPoint(icon.width()/2,icon.height()/2));
            drag->drag();
         }
         else
         {
            QListView::contentsMouseMoveEvent(event);
         }
      }
      else
      {
         QListView::contentsMouseMoveEvent(event);
      }
   }
   else
   {
      QListView::contentsMouseMoveEvent(event);
   }
}

void CatalogManagerView::readMarker()
{
   KConfig* config = KGlobal::config();
   KConfigGroupSaver cs(config,"CatalogManager");

   _markerList = config->readListEntry("Marker");
}

void CatalogManagerView::saveMarker()
{
   KConfig* config = KGlobal::config();
   KConfigGroupSaver cs(config,"CatalogManager");

   config->writeEntry("Marker",_markerList);
}




void CatalogManagerView::processEnded(KProcess* proc)
{
   _pendingProcesses.removeRef(proc);
}


void CatalogManagerView::showOutput(KProcess*, char *buffer, int buflen)
{
   QCString output(buffer,buflen+1);

   _logView->append(output);
}




CatManListItem::CatManListItem(QListViewItem* parent,QString fullPath,QString fullPotPath,QString package)
     : QListViewItem(parent)
{
   init(fullPath,fullPotPath,package);
}

CatManListItem::CatManListItem(QListView* parent,QString fullPath,QString fullPotPath)
     : QListViewItem(parent)
{
   _primary=QFileInfo(fullPath);
   _template=QFileInfo(fullPotPath);
   _package="/";
   _type=Dir;
   _marked=false;

   _primary.setCaching(false);
   _template.setCaching(false);

   setText(COL_NAME,i18n("Message Catalogs"));
   setPixmap(COL_NAME,KGlobal::iconLoader()->loadIcon("folder",KIconLoader::Small));
}


void CatManListItem::init(const QString& fullPath, const QString& fullPotPath, const QString& package)
{
   _primary=QFileInfo(fullPath);
   _template=QFileInfo(fullPotPath);
   _package=package;
   _marked=false;

   _primary.setCaching(false);
   _template.setCaching(false);

    // set
    _lastUpdated=QDate(1900,1,1);

    update(parent()->isOpen());
}

void CatManListItem::setMarked(bool on)
{
   if(on)
   {
      setPixmap(COL_MARKER,BarIcon("flag"));
   }
   else
   {
      setPixmap(COL_MARKER,QPixmap());
   }

   _marked=on;
}

void CatManListItem::setOpen(bool open)
{
    QListViewItem::setOpen(open);

    if(open && _type==Dir)
    {
       QPixmap icon=KGlobal::iconLoader()->loadIcon("folder_open",KIconLoader::Small);

       if(!_template.exists())
       {
          paintExclamation(&icon);
       }

       setPixmap(COL_NAME,icon);

       CatManListItem * myChild = (CatManListItem*)firstChild();
       while( myChild )
       {
         myChild->checkUpdate();
         myChild = (CatManListItem*)myChild->nextSibling();
       }
    }
    else
    {
       QPixmap icon=KGlobal::iconLoader()->loadIcon("folder",KIconLoader::Small);

       if(!_template.exists())
       {
          paintExclamation(&icon);
       }

       setPixmap(COL_NAME,icon);
    }

}

QStringList CatManListItem::allChildrenList(bool onlyFiles) const
{
   QStringList childrenList;

   CatManListItem * myChild = (CatManListItem*)firstChild();
   while( myChild )
   {
      QString name=myChild->package();

      if(myChild->isFile())
      {
         childrenList.append(name);
      }
      else if(myChild->isDir())
      {
         if(!onlyFiles)
            childrenList.append(name);

         childrenList+=myChild->allChildrenList(onlyFiles);
      }

      myChild = (CatManListItem*)myChild->nextSibling();
   }

   return childrenList;
}


QStringList CatManListItem::contentsList(bool onlyFiles) const
{
   QStringList childList;

   CatManListItem * myChild = (CatManListItem*)firstChild();
   while( myChild )
   {
      QString name=myChild->package();

      if(onlyFiles)
      {
         if(myChild->isFile())
         {
            childList.append(name);
         }
      }
      else
      {
         childList.append(name);
      }

      myChild = (CatManListItem*)myChild->nextSibling();
   }

   return childList;
}


void CatManListItem::checkUpdate()
{
   if(_primary.isFile() && _lastUpdated < _primary.lastModified())
   {
      update();
   }
   else if(_template.isFile()  && _lastUpdated < _template.lastModified())
   {
      update();
   }


}

QString CatManListItem::key(int col, bool) const
{
   // show directories first
   QString key=text(col);

   if(col==COL_NAME)
   {
      if(_type==Dir)
      {
         key="a"+key;
      }
      else
      {
         key="b"+key;
      }
   }
   // fuzzy, untranslated, total
   else if(col==COL_FUZZY || col ==COL_TOTAL || col==COL_UNTRANS)
   {
      key=key.rightJustify(10,'0');
   }


   return key;
}

void CatManListItem::update(bool showPoInfo,bool includeChildren)
{
    if(_package!="/") // don't update root item
    {
       if(_primary.exists())
       {
          if(_primary.isDir())
          {
             QDir dir=_primary.dir();
             setText(COL_NAME,dir.dirName());
             setSelectable(false);
             _type=Dir;
             // check if the same directory exists also in the template directory
             if(_template.isDir())
             {
                setPixmap(COL_NAME,KGlobal::iconLoader()->loadIcon("folder",KIconLoader::Small));
             }
             else
             {
                QPixmap folder=KGlobal::iconLoader()->loadIcon("folder",KIconLoader::Small);
                paintExclamation(&folder);

                setPixmap(COL_NAME,folder);
             }
          }
          else
          {
             _type=File;
              QString name=_primary.fileName();
              setText(COL_NAME,name.left(name.length()-3));

             if(showPoInfo)
             {
                _lastUpdated=QDateTime::currentDateTime();

                PoInfo poInfo;
                QPixmap icon;
                if(Catalog::info(_primary.absFilePath(),poInfo)==Catalog::OK)
                {
                    if(poInfo.fuzzy==0 && poInfo.untranslated==0)
                    {
                        //setPixmap(0,KGlobal::iconLoader()->loadIcon("ok",KIconLoader::Small));
                        icon=BarIcon("ok");
                    }
                    else
                    {
                       //setPixmap(0,KGlobal::iconLoader()->loadIcon("needwork",KIconLoader::Small));
                       icon=BarIcon("needwork");
                    }
                    setText(COL_FUZZY,QString::number(poInfo.fuzzy));
                    setText(COL_UNTRANS,QString::number(poInfo.untranslated));
                    setText(COL_TOTAL,QString::number(poInfo.total));
                    setText(COL_REVISION,poInfo.revision);
                    setText(COL_TRANSLATOR,poInfo.lastTranslator);
                }
                else
                {
                    // setPixmap(0,KGlobal::iconLoader()->loadIcon("unknown",KIconLoader::Small));
                    icon=BarIcon("unknown");
                }

                if(!_template.exists())
                {
                   paintExclamation(&icon);
                }

                setPixmap(COL_NAME,icon);
             }
          }
       }
       else if(_template.exists())
       {
          if(_template.isDir())
          {
             QDir dir=_template.dir();
             setText(COL_NAME,dir.dirName());
             setSelectable(false);
             _type=Dir;

             setPixmap(COL_NAME,KGlobal::iconLoader()->loadIcon("folder",KIconLoader::Small));
          }
          else
          {
             _type=File;
             QString name=_primary.fileName();
             setText(COL_NAME,name.left(name.length()-3));
             // setPixmap(0,KGlobal::iconLoader()->loadIcon("missing",KIconLoader::Small));
             setPixmap(COL_NAME,BarIcon("missing"));

             if(showPoInfo)
             {
                _lastUpdated=QDateTime::currentDateTime();

                PoInfo poInfo;
                if(Catalog::info(_template.absFilePath(),poInfo)==Catalog::OK)
                {
                   setText(COL_TOTAL,QString::number(poInfo.total));
                }
             }
          }
       }
       else
       {
          kDebugFatal(KBABEL_CATMAN,"wether po nor pot exists: %s"
               ,_package.ascii());
       }
    }

    if(includeChildren)
    {
       CatManListItem * myChild = (CatManListItem*)firstChild();
       while( myChild )
       {
          myChild->update(showPoInfo,includeChildren);
          myChild = (CatManListItem*)myChild->nextSibling();
       }
    }
}

bool CatManListItem::hasPo() const
{
   return _primary.exists();
}

bool CatManListItem::hasPot() const
{
   return _template.exists();
}

int CatManListItem::fuzzy() const
{
   bool success;
   int number=text(COL_FUZZY).toInt(&success);
   if(!success)
      number=0;

   return number;
}

int CatManListItem::untranslated() const
{
   bool success;
   int number;
   if( !hasPo() )
   {
      number=total();
   }
   else
   {
      number=text(COL_UNTRANS).toInt(&success);
      if(!success)
         number=0;
   }

   return number;
}

int CatManListItem::total() const
{
   bool success;
   int number=text(COL_TOTAL).toInt(&success);
   if(!success)
      number=0;

   return number;
}

bool CatManListItem::isDir() const
{
   return type()==Dir;
}

bool CatManListItem::isFile() const
{
   return type()==File;
}

QString CatManListItem::poFile() const
{
   return _primary.absFilePath();
}

QString CatManListItem::potFile() const
{
   return _template.absFilePath();
}

QString CatManListItem::package(bool rootSlash) const
{
   if(rootSlash)
      return _package;
   else
   {
      return _package.right(_package.length()-1);
   }
}

QString CatManListItem::name() const
{
   int index = _package.findRev("/");
   return _package.right(_package.length()-index-1);
}

void CatManListItem::paintExclamation(QPixmap* pixmap)
{
   int width=pixmap->width();
   int height=pixmap->height();

   int diameter=QMIN(width,height);

   QBitmap mask=pixmap->createHeuristicMask();

   QPainter mp(&mask);
   mp.setPen(QPen(Qt::color1,1));
   mp.drawEllipse(width-diameter,height-diameter,diameter,diameter);

   QPainter p(pixmap);
   p.setPen( QPen(red,1) );
   p.drawEllipse(width-diameter,height-diameter,diameter,diameter);

   pixmap->setMask(mask);

}
















CatalogManager::CatalogManager(CatManSettings settings)
                 :KDialogBase(0,0,false,i18n("Catalog Manager"),Close|Help|User1|User2|User3)
{
   _catalogManager=new CatalogManagerView(this,"catalog manager");
   connect(_catalogManager,SIGNAL(settingsChanged(CatManSettings))
            ,this,SIGNAL(settingsChanged(CatManSettings)));
   connect(_catalogManager,SIGNAL(openFile(QString))
           ,this,SLOT(openFile(QString)));
   connect(_catalogManager,SIGNAL(openTemplate(QString,QString))
           ,this,SLOT(openTemplate(QString,QString)));

   connect(this,SIGNAL(user2Clicked()),_catalogManager,SLOT(statistics()));
   connect(this,SIGNAL(user3Clicked()),_catalogManager,SLOT(checkSyntax()));

   setButtonText(User1,i18n("C&onfigure..."));
   setButtonText(User2,i18n("&Statistics"));
   setButtonText(User3,i18n("S&yntax"));

   setMainWidget(_catalogManager);

   _prefWidget=0;
   _prefDialog=0;

   setSettings(settings);

   restoreView();

}

CatalogManager::~CatalogManager()
{
   saveView();
}


CatManSettings CatalogManager::settings() const
{
   return _catalogManager->settings();
}

void CatalogManager::setSettings(CatManSettings newSettings)
{
   _catalogManager->setSettings(newSettings);

   _openNewWindow=newSettings.openWindow;
}

void CatalogManager::setPreferredWindow(KBabel* window)
{
   _preferredWindow = window;
}

void CatalogManager::updateFile(QString fileWithPath)
{
   _catalogManager->updateFile(fileWithPath);
}

void CatalogManager::openFile(QString filename)
{
   if(_preferredWindow)
   {
      _preferredWindow=(KBabel*)(KBabel::memberList->first());
   }

   if(!_openNewWindow)
   {
      KWin::setActiveWindow(_preferredWindow->winId());

      if(_preferredWindow->isMinimized())
         _preferredWindow->showNormal();


      _preferredWindow->setActiveWindow();
      _preferredWindow->raise();

   }

   _preferredWindow->open(filename,_openNewWindow);
}

void CatalogManager::openTemplate(QString openFilename,QString saveFilename)
{
   if(!_preferredWindow)
   {
      _preferredWindow=(KBabel*)(KBabel::memberList->first());
   }

   if(!_openNewWindow)
   {
      KWin::setActiveWindow(_preferredWindow->winId());

      if(_preferredWindow->isMinimized())
         _preferredWindow->showNormal();

      _preferredWindow->setActiveWindow();
      _preferredWindow->raise();
   }

   _preferredWindow->openTemplate(openFilename,saveFilename,_openNewWindow);
}

void CatalogManager::slotHelp()
{
   kapp->invokeHTMLHelp("","catalogmanager");
}

void CatalogManager::slotUser1()
{
   if(!_prefDialog)
   {
      _prefDialog = new KDialogBase(this,0,true,i18n("Catalog Manager Preferences")
                     ,Ok|Cancel|Help);
      _prefDialog->setHelp("catalogmanager","");

      _prefWidget=new CatManPreferences(_prefDialog);

      _prefDialog->setMainWidget(_prefWidget);
   }

   _prefWidget->setSettings(_catalogManager->settings());
   if(_prefDialog->exec())
   {
      setSettings(_prefWidget->settings());
      emit settingsChanged(_prefWidget->settings());
   }
}


void CatalogManager::saveView() const
{
    KConfig* config = KGlobal::config();
    KConfigGroupSaver cs(config,"CatalogManager");

    config->writeEntry("size",size());
}


void CatalogManager::restoreView()
{
    KConfig* config = KGlobal::config();
    KConfigGroupSaver cs(config,"CatalogManager");

    QSize defaultSize(400,400);
    setInitialSize(config->readSizeEntry("size",&defaultSize));
}


